class Tab_SpeechBinder extends Tab_ControlSettings;

var transient bool bNoMatureLanguage;
var localized string SpeechLabel;

function InitComponent(GUIController MyController, GUIComponent MyOwner)
{
	GUILabel(Controls[4]).Caption = SpeechLabel;
	Controls[7].bVisible = false; // Hide 'reset' button

	LoadSpeechCommands();

	bNoMatureLanguage = MyController.ViewportOwner.Actor.bNoMatureLanguage;

	Super.Initcomponent(MyController, MyOwner);
}

function AddCommand(bool IsSection, string KeyLabel, string Alias)
{
	if( NumBindings == ArrayCount(Bindings) )
	{
		Log("Not enough space!!");
		return;
	}

	Bindings[NumBindings].bIsSectionLabel = IsSection;
	Bindings[NumBindings].KeyLabel = KeyLabel;
	Bindings[NumBindings].Alias = Alias;

	Labels[NumBindings] = KeyLabel;
	NumBindings++;
}

// We pass in the controller here
function LoadSpeechCommands()
{
	local int i;
		
	// Empty the control binds
	NumBindings = 0;

	//////////////////////////////// ACKNOWLEDGEMENTS /////////////////////////////////////////////
	AddCommand(true, class'ExtendedConsole'.default.SMStateName[1], "");

	for(i=0; i<class'xVoicePack'.default.numAcks; i++)
	{
		AddCommand(false, class'xVoicePack'.default.AckString[i], "speech ACK "$i);
	}

	//////////////////////////////// FRIENDLY FIRE /////////////////////////////////////////////
	AddCommand(true, class'ExtendedConsole'.default.SMStateName[2], "");

	for(i=0; i<class'xVoicePack'.default.numFFires; i++)
	{
		if( class'xVoicePack'.default.FFireAbbrev[i] != "" )
			AddCommand(false, class'xVoicePack'.default.FFireAbbrev[i], "speech FRIENDLYFIRE "$i);
		else
			AddCommand(false, class'xVoicePack'.default.FFireString[i], "speech FRIENDLYFIRE "$i);
	}

	//////////////////////////////// ORDERS /////////////////////////////////////////////
	AddCommand(true, class'ExtendedConsole'.default.SMStateName[3], "");

	for(i=0; i<9; i++)
	{
		if( class'xVoicePack'.default.OrderString[i] == "" )
			continue;

		if(class'xVoicePack'.default.OrderAbbrev[i] != "")
			AddCommand(false, class'xVoicePack'.default.OrderAbbrev[i], "speech ORDER "$i);
		else
			AddCommand(false, class'xVoicePack'.default.OrderString[i], "speech ORDER "$i);
	}

	//////////////////////////////// OTHER /////////////////////////////////////////////
	AddCommand(true, class'ExtendedConsole'.default.SMStateName[4], "");

	for(i=0; i<ArrayCount(class'xVoicePack'.default.OtherString); i++)
	{
		if( class'xVoicePack'.default.OtherString[i] == "" )
			continue;

		if( class'xVoicePack'.default.OtherAbbrev[i] != "" )
			AddCommand(false, class'xVoicePack'.default.OtherAbbrev[i], "speech OTHER "$i);
		else
			AddCommand(false, class'xVoicePack'.default.OtherString[i], "speech OTHER "$i);
	}

	//////////////////////////////// LOAD TAUNTS /////////////////////////////////////////////
	// NB. This wont get species-specific taunts. Oh well :)
	AddCommand(true, class'ExtendedConsole'.default.SMStateName[5], "");

	for(i=0; i<class'xVoicePack'.default.numTaunts; i++)
	{
		if( class'xVoicePack'.default.MatureTaunt[i] == 1 && bNoMatureLanguage )
			continue;

		if( class'xVoicePack'.default.TauntAbbrev[i] != "" )
			AddCommand(false, class'xVoicePack'.default.TauntAbbrev[i], "speech TAUNT "$i);
		else
			AddCommand(false, class'xVoicePack'.default.TauntString[i], "speech TAUNT "$i);
	}
}

defaultproperties
{
	SpeechLabel="Phrase"
}